Program Mooncal;
{Jarmo Lammi 10.2.2000
 Computes Moon position with perturburation corrections
 for current time and date.
 Calculates the geocentric position of Moon in terms of RA and declination.
 HA is calculated, RA and declin. are converted to topocentric values.
 Finally, the topocentric RA and declination are converted
 to azimuth and altitude.20.2.2000 added check for ascending/descending Moon phase}

{$N+}
USES CRT,DOS;
CONST dgr = CHR(248);
rads = Pi/180.0;
pt = '.';

VAR vv,kk,pp,vd:  WORD;
hr,mn,sc,tc:      Word;
jd:               Longint;
latecl,longecl,latecl2,longecl2: double;
dM,dN,di,dw,dE0,dE1,a,e:         double;

x,y,r,v,r2,z:                    double;
xh,yh,zh:                        double;
xhor,yhor,zhor:                  double;
xe,ye,ze:                        double;
xecl,yecl,zecl,oblec:            double;
RA,declin,HA,topRA,topDecl:        double;
topHA,azimuth,sw,Ls,Lm,Ms,Mm,D,F:  double;
altit,sunaz:                             double;
cEv,cVa,cYr,clon4,clat,clat2,cR:   double;
mpar,gclat,rho,gaux:               double;
SIDTIME,UT,LON,LAT,TZone:          Real;
GMST0,frlumin: Real;
amsg:   String;
FUNCTION TAN(x:real):real;
BEGIN TAN:=SIN(x)/COS(x) END;

 Function Jday(y,m,d:Word): Longint;
 VAR rd: Real;

 Begin

{Calculate Julian Day number }
        rd:= 367.0*vv;
        rd:= rd + 1.0*(-(7*(y + ((m+9) Div 12))) Div 4 + (275*m) Div 9);
        rd:= rd + 1.0*d - 730530.0;
 Jday:= Longint(Round(rd))
 End;

 Function rev(x: double): double;
 {Normalize degrees to range 0 ... 360}
 var rv: double;
 Begin
   rv:= x - trunc(x/360.0)*360.0;
   if (rv<0.0) then rv:= rv + 360.0;
   rev:= rv;
 End;

 Function dEn(e0,m,e:double): double;
{Iteration of E }
var e1: double;
Begin
     e1:= e0 - (e0 - (180/Pi)*e*sin(rads*e0) - m)/(1.0 - e*cos(rads*e0));
     dEn:= e1
End;

Function atan2(y,x: double): double;
Begin
if (x=0.0) then
   Begin
        if (y=0.0) then Exit
        else if (y>0.0) then atan2:= 0.5*Pi
        else atan2:= -0.5*Pi
   End
else
    Begin
         if (x>0.0) then atan2:= arctan(y/x)
         else if (x<0.0) then
         Begin
              if (y>=0.0) then atan2:= arctan(y/x) + Pi
              else atan2:= arctan(y/x) - Pi;
         End;
    End;
End;


Begin   {MAIN}

        TextBackGround(Blue); TextColor(White); ClrScr;
        ClrScr;
        Write('Give your latitude: '); Readln(LAT);
        Write('Give your longitude: '); Readln(LON);
        Write('Give your timezone: '); Readln(TZone);
        TextBackGround(White);
        TextColor(Blue);
        ClrScr;

        GetDate(vv,kk,pp,vd);
{We use the current date and time};

        jd:= Jday(vv,kk,pp);

        GetTime(hr,mn,sc,tc);
        {UT corresponding to timezone}
        UT:= 1.0*hr + mn/60.0 + sc/3600.0 -TZone;
        GoToXY(1,1);
        Write('Date: ',pp,pt,kk,pt,vv);
        Write(' Julian day #',jd);

        Write('  Time ',hr,':',mn,':',sc);
{Now we initialize the orbital elements of Moon:}
     dN := 125.1228 -0.0529538083*jd; {Long asc. node}
     di := 5.1454; {Inclination}
     dw := 318.0634 + 0.1643573223*jd; {Arg. of perigee}
     a := 60.2666; {Mean distance}
     e := 0.054900; {Eccentricity}
     dM := 115.3654 + 13.0649929509*jd; {Mean anomaly}
     dM:= rev(dM); dw:= rev(dw);
     {Eccentric anomaly first approximation}
     dE0:= dM + (180.0/Pi)*e*sin(rads*dM)*(1.0 + e*cos(rads*dM));
     dE1:= dEn(dE0,dM,e); {Iteration of E}
     dE1:= dEn(dE1,dM,e);

{Compute rectangular coordinates of the lunar orbit}
     x:= a*(cos(rads*dE1) - e);
     y:= a*sqrt(1.0 - e*e)*sin(rads*dE1);
     r:= sqrt(x*x + y*y);
     v:= rev(atan2(y,x)/rads);
{Moon's position in ecliptic coordinates}
        xecl:= r*(cos(rads*dN)*cos(rads*(v+dw)) - sin(rads*dN)*sin(rads*(v+dw))*cos(rads*di));
        yecl:= r*(sin(rads*dN)*cos(rads*(v+dw)) + cos(rads*dN)*sin(rads*(v+dw))*cos(rads*di));
        zecl:= r*sin(rads*(v+dw))*sin(rads*di);
        longecl:= rev(atan2(yecl,xecl)/rads);
        latecl:= atan2(zecl,sqrt(xecl*xecl + yecl*yecl))/rads;
{Now we shall compute perturbations for higher accuracy}
{Ls Sun's mean longitude
Ms Sun's mean anomaly
Lm Moon's mean longitude }
        sw:= 282.9404 + 4.70935E-5*jd;
        Ms:= 356.047 + 0.9856002585*jd;
        Ls:= rev(sw + Ms); Ms:=rev(Ms);
        Lm:= dN + dw + dM; Lm:= rev(Lm);


        D:= Lm - Ls;
        F:= Lm - dN;
        cEv:= -1.274*sin(rads*(dM-2.0*D));
        cVa:=  0.658*sin(2.0*rads*D);
        cYr:= -0.186*sin(rads*Ms);
        clon4:= -0.059*sin(2.0*rads*(dM-D))
                -0.057*sin(rads*(dM-2.0*D+Ms))
                +0.053*sin(rads*(dM+2.0*D))
                +0.046*sin(rads*(2.0*D-Ms))
                +0.041*sin(rads*(dM-Ms))
                -0.035*sin(rads*D)
                -0.031*sin(rads*(dM+Ms))
                -0.015*sin(2.0*rads*(F-D))
                +0.011*sin(rads*(dM-4.0*D));

        clat:= -0.173*sin(rads*(F-2.0*D));
        clat2:= -0.055*sin(rads*(dM - F -2.0*D))
                -0.046*sin(rads*(dM + F -2.0*D))
                +0.033*sin(rads*(F+2.0*D))
                +0.017*sin(rads*(2.0*dM + F));

        cR:= -0.58*cos(rads*(dM-2.0*D)) -0.46*cos(rads*(2.0*D));

        {Corrected longitude}
        longecl2:= longecl + cEv + cVa + cYr + clon4;
        {Corrected latitude}
        latecl2:=  latecl + clat + clat2;

        {Corrected distance}
        r2:= r + cR;
        {Geocentric perturbed coordinates}
        xh:= r2*cos(rads*longecl2)*cos(rads*latecl2);
        yh:= r2*sin(rads*longecl2)*cos(rads*latecl2);
        zh:= r2*sin(rads*latecl2);
        xe:= xh;
        oblec:= 23.4393 - 3.563E-7*jd;
        ye:= yh*cos(rads*oblec) - zh*sin(rads*oblec);
        ze:= yh*sin(rads*oblec) + zh*cos(rads*oblec);
        RA:= rev(atan2(ye,xe)/rads);
        declin:= atan2(ze,sqrt(xe*xe+ye*ye));
        declin:= declin/rads;

        {Moon hour angle HA calculated here}

        GMST0:= Ls/15.0 + 12.0;
        SIDTIME := GMST0 + UT + LON/15.0;

        HA := rev(15.0*SIDTIME - RA);
        mpar:= arctan(1.0/r2); {Moon parallax at distance r2}
        {Take account for the flattening of Earth}
        gclat:= LAT - 0.1924*sin(2.0*rads*LAT);
        rho:= 0.99883 + 0.00167*cos(2.0*rads*LAT);
        gaux:= arctan(tan(rads*gclat)/cos(rads*HA));
        topRA:= RA -mpar*rho*cos(rads*gclat)
                *sin(rads*HA)/cos(rads*declin)/rads;
        topDecl:= declin -mpar*rho*sin(rads*gclat)
                  *sin(gaux - declin*rads)/sin(gaux)/rads;
        {Calculate topocentric HA}
        topHA:= rev(15.0*SIDTIME - topRA);
        {The final phase of calculations beginning}
        x:= cos(rads*topHA)*cos(rads*topDecl);
        y:= sin(rads*topHA)*cos(rads*topDecl);
        z:= sin(rads*topDecl);
        xhor:= x*sin(rads*gclat) -z*cos(rads*gclat);
        yhor:= y;
        zhor:=x*cos(rads*gclat) + z*sin(rads*gclat);
        azimuth:= atan2(yhor,xhor) + Pi;
        altit:= atan2(zhor,sqrt(xhor*xhor+yhor*yhor));
        sunaz:= 15.0*UT +  LON; sunaz:= rads*sunaz;
        frlumin:= (sunaz - azimuth);
        if (frlumin>Pi) then frlumin:= 2.0*Pi - frlumin;
        if (frlumin <-Pi) then frlumin:= frlumin + Pi;
        frlumin:= frlumin/Pi;

 amsg:= ' descending';

 if (sunaz > Pi) and (azimuth < Pi) and (sunaz-azimuth < Pi) and (sunaz - azimuth > 0) then amsg:= ' ascending';
 if (sunaz < Pi) and (azimuth > Pi) and (azimuth-sunaz > Pi) and (sunaz - azimuth < 0) then amsg:= ' ascending';

        GoToXY(1,3);

        Writeln('Location:');
        Writeln('Latitude = ',LAT:3:2,dgr,
        ' Longitude = ',LON:3:2,dgr,' Timezone ',TZone:2:0);

        TextColor(red);
        Writeln;
        Writeln('MOON POSITION COMPUTED:');
        TextColor(black);
        Writeln('Eclip. longit= ',longecl:4:4,dgr,
        ' Eclip. latit= ',latecl:4:4,dgr);


        Write('Corr. latitude= ',latecl2:4:4,dgr);
        Writeln(' and longitude= ',longecl2:4:4,dgr);
        Write('Corrected distance= ',r2:3:4);
        Writeln(' Earth radii');

        Writeln('RA= ',RA:4:4,' Declin= ',declin:4:4,dgr);
{up to this point exactly the right test values}
        Writeln('SIDTIME=',SIDTIME:3:5,' hours, HA = ',
        (HA/15.0):3:2,' h = ',HA:3:4,dgr);
        Write('Parallax of Moon is ',(mpar/rads):1:4,dgr);
        Writeln(' gclat = ',gclat:2:2,dgr,' rho = ',rho:1:4);
        Write('topoc RA = ',topRA:3:4,dgr,
        '  topoc Declin = ',topDecl:3:4,dgr);
        Writeln(' Topoc HA = ',topHA:3:4,dgr);
        TextColor(red);
        Write('Azimuth= ',azimuth/rads:4:2,dgr);
        Write(' Altitude= ',altit/rads:3:2,dgr);
        Write(' Fraction illuminated= ',abs(frlumin):1:2);
        Writeln(amsg);
        Writeln('Sun azimuth: ',sunaz/rads:4:2,dgr)
End.
